function outputParameters = VidaGetChannelSubFactor(inputParameters)

%This function takes a set of files from two channels of simultaneously
%acquired data (both sets of files must be the same size)
%and calculates a channel subtraction factor
%the main channel file has some bleed through from the bleed channel
%the bleed channel must be "pure".
%Channel subtraction factor is determined by:
%   sum projecting across a set of z-frames in both channels
%   making a mask of the brightest n% of the pixels in the bleed channel
%   subtracting the channels with various multiplying factors
%   comparing the mode of the non-masked region to the masked region
%   when the masked region mode drops below the non-masked region mode
%       then the correct subtraction factor has been obtained

processedDataFolder = inputParameters.processedDataFolder;                        %folder in which data is stored
mainFileString = inputParameters.mainFileString;                %e.g., 'NEUN-*' or 'Stacked-FITC.tif'
bleedFileString = inputParameters.bleedFileString;              %e.g., 'FITC-*' or 'Stacked-HORD.tif'
bleedMaskTopFraction = inputParameters.bleedMaskTopFraction;    %brightest percentage of the projected frames to mask
maxSubtractionFactor = inputParameters.maxSubtractionFactor;    %largest factor to use
numFactors = inputParameters.numFactors;                        %number of factors to step through
skipFirstNFiles = inputParameters.skipFirstNFiles;              %skip the first N matchingfiles in the directory
skipLastNFiles = inputParameters.skipLastNFiles;                %skip the last N matching files in the directory
subBlockSize = inputParameters.subBlockSize;                    %if files larger than any of these dimensions, then subblock
startFrameFraction = inputParameters.startFrameFraction;        %0.5 means start with frame halfway down stack
numFramesToProject = inputParameters.numFramesToProject;        %sum project before evaluatio
fitPolynomialDegree = inputParameters.fitPolynomialDegree;      %degree of polynomial to curve fit
 
 
factorList = linspace(0,maxSubtractionFactor,numFactors);

errorFlag = 0;
mainFileInfo = dir(fullfile(processedDataFolder,mainFileString));
bleedFileInfo = dir(fullfile(processedDataFolder,bleedFileString));
if size(mainFileInfo,1)~=size(bleedFileInfo,1),errorFlag = 1;end
subtractionFactor = [];
iterCounter = 0;

totalNumFiles = size(mainFileInfo,1);
numFilesToProcess = totalNumFiles - skipFirstNFiles - skipLastNFiles;
for fileIter = 1:totalNumFiles,
    mainFileName = fullfile(processedDataFolder,mainFileInfo(fileIter).name);
    bleedFileName= fullfile(processedDataFolder,bleedFileInfo(fileIter).name);
    
    mainFileImfInfo = imfinfo(mainFileName);
    bleedFileImfInfo = imfinfo(bleedFileName);
    if length(mainFileImfInfo)~=length(bleedFileImfInfo), errorFlag = 1; end
    if mainFileImfInfo(1).Width~=bleedFileImfInfo(1).Width, errorFlag = 1; end
    if mainFileImfInfo(1).Height~=bleedFileImfInfo(1).Height, errorFlag = 1; end

    if errorFlag == 1,
        disp('Error in Channel Subtraction Files');
        keyboard
    end

    %break into smaller blocks if necessary
    numYBlocks = ceil(mainFileImfInfo(1).Height./subBlockSize(1));
    numXBlocks = ceil(mainFileImfInfo(1).Width./subBlockSize(2));
    numZBlocks = ceil(length(mainFileImfInfo)./subBlockSize(3));
    totalNumBlocks = numYBlocks.*numXBlocks.*numZBlocks.*numFilesToProcess;
    
    xSize = mainFileImfInfo(1).Width;
    ySize = mainFileImfInfo(1).Height;
    zSize = length(mainFileImfInfo);
   
    yRange = round(linspace(1,ySize,numYBlocks+1));
    xRange = round(linspace(1,xSize,numXBlocks+1));
    zRange = round(linspace(1,zSize,numZBlocks+1));
 
    
    for zBlockIter = 1:numZBlocks,
        zStart = round((zRange(zBlockIter+1) - zRange(zBlockIter))*startFrameFraction)+zRange(zBlockIter);
        zStop = zStart + numFramesToProject;
        mainBlock = loadmovie(mainFileName,[0,0,0,0,zStart,zStop]);
        bleedBlock = loadmovie(bleedFileName,[0,0,0,0,zStart,zStop]);
        for xBlockIter = 1:numXBlocks,
            for yBlockIter = 1:numYBlocks,
                iterCounter = iterCounter+1;
                display(['Calculating Factor on Block ',num2str(iterCounter),' of ',...
                    num2str(totalNumBlocks),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
                mainSubBlock = mainBlock(yRange(yBlockIter):yRange(yBlockIter+1),...
                    xRange(xBlockIter):xRange(xBlockIter+1),:);
                bleedSubBlock = bleedBlock(yRange(yBlockIter):yRange(yBlockIter+1),...
                    xRange(xBlockIter):xRange(xBlockIter+1),:); 
                mainFrame = sum(double(mainSubBlock),3);
                bleedFrame = sum(double(bleedSubBlock),3);
                bleedSorted = sort(bleedFrame(:));
                numPixels = numel(bleedSorted);
                cutoffPixel = numPixels - floor(numPixels*bleedMaskTopFraction);
                bleedMaskThresh = bleedSorted(cutoffPixel);
                bleedMask = bleedFrame>bleedMaskThresh;
                whereMask = find(bleedMask>0);
                whereNotMask = find(bleedMask==0);
                for iterFactor = 1:length(factorList);
                     currentFactor = factorList(iterFactor);
                     subtractedFrame = mainFrame - currentFactor.*bleedFrame;
                     subtractedMasked = subtractedFrame(whereMask);
                     subtractedNotMasked = subtractedFrame(whereNotMask);
                     histXRange = [-1000:1000];
                     h = hist(subtractedMasked,histXRange);
                     h(1) = 0;h(end) = 0;
                     maskedModeIndex = find(h == max(h));
                     maskedModeValue(iterFactor) = histXRange(maskedModeIndex(1));
                     maskedMedianValue(iterFactor) = median(subtractedMasked);
                     h = hist(subtractedNotMasked,histXRange);
                     h(1) = 0;h(end) = 0;
                     notMaskedModeIndex = find(h == max(h));
                     notMaskedModeValue(iterFactor) = histXRange(notMaskedModeIndex(1));
                     notMaskedMedianValue(iterFactor) = median(subtractedNotMasked);
                end%for iterFactor = 1:length(factorList);
                modeComparison = maskedModeValue - notMaskedModeValue;

                [p,S] = polyfit(factorList,modeComparison,fitPolynomialDegree);
                f = polyval(p,factorList);
                closestToZero = find(abs(f) == min(abs(f)));
                subtractionFactor(iterCounter) = factorList(closestToZero);
    
                
            end% for yBlockIter = 1:numYBlocks,
        end% for xBlockIter = 1:numXBlocks,
    end% for zBlockIter = 1:numZBlocks,
end%for fileIter = 1:numFiles,
    
medianSubtractionFactor = median(subtractionFactor);
display(['medianSubtractionFactor = ',num2str(medianSubtractionFactor)]);

outputParameters = inputParameters;
outputParameters.medianSubtractionFactor = medianSubtractionFactor;



    
