function [GapConnectionStruct] = LocalThresholdRelaxationConnection(VesselRepresentations)

%% VesselRepresentations is a structure including:
% RawVesselVolumeTifFname          : raw vessel volume tif full filename (FITC - a DAPI)
% EnhancedVesselVolumeTifFname     : rod match filter-enhanced vessel volume tif full filename
% IsotropicVesselMask              : isotropic vessel mask's tif full filename
% VectorizedMV3DFilename           : amira's vectorized vessel .mv3d full filename
% GapConnectionParameters          : all parameters specifying how to connect gaps
%    prmts.boundBoxMinDist         : an endpoint must be at least this distance from bounding box to connect
%    prmts.boundBoxDimXYZ          : the range of x,y,z for the vessel volume data
%    prmts.maxGapDist_um           : 1pxl = 1um
%    prmts.minGapDist_um           : 0 (set to some nonzero value for studies of gap length)
%    hopislands                    : {0,1} (toggles whether islands can be connection candidates)
%    BBmargin                      : the margin of the bounding box about an endpoint to find connection candidates
%    threshparameters.minvvol      : parameter to JointThresholdVesselMasks indicating smallest volume for vessel segment
%    threshparameters.morphfiltsz  : parameter to JointThresholdVesselMasks indicating smallest volume for vessel segment

%% GapConnectionStruct
% GapConnectionStruct.strandSet            : The .mv3d file cast as a matlab data structure
% GapConnectionStruct.allconnectionptpairs : Indicates what pt indices in strandSet should be connected
% GapConnectionStruct.connectiontypes      : TR or NN (for Threshold Relaxation or Nearest Neighbor); indicates
%                                          : whether the connection pair was connected via a simple nearest neighbor
%                                          : method or the threshold relaxation method.
% GapConnectionStruct.ThreshRelax          : Contains 2 fields, "BridgingStrands" and "BridgingConfidences"
%                                          : Bridging strands is a cell array of ordered lists of 3D centerline coordinates
%                                          : Bridging confidences is the corresponding scalar confidence per strand

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

%% Load, Parse, and Compatibly index .mv3d file in matlab
strandSet = amira2mat_parseFile(VesselRepresentations.VectorizedMV3DFilename);
strandSet = amira2mat_buildStrandAdjacency(strandSet);
%% One can connect small gaps by pairing endpoints and testing for proximity and directionality
[strandSet_jk connections_jk connectionsretest_jk] = PairNearestNeighborEndpoints(strandSet,VesselRepresentations.GapConnectionParameters.prmts);
%% Get centerpoints of islands in mask
[IslandCenterPoints] = GetIslandPointsFromMask(VesselRepresentations.IsotropicVesselMaskMatFname,VesselRepresentations.VectorizedMV3DFilename);
IslandCenterPoints = [IslandCenterPoints(:,2),IslandCenterPoints(:,1),IslandCenterPoints(:,3)];
%% Load entire raw volume
Vvecinfo = imfinfo(VesselRepresentations.RawVesselVolumeMatFname);
Vvecsz = [Vvecinfo(1).Width, Vvecinfo(1).Height, numel(Vvecinfo)];

save ASWIdebug
%% Augment strands with island centers
augmentedStrandSet = AugmentStrandsWithIslands(strandSet_jk,IslandCenterPoints);
% compensate strand locations for 0-based to 1-based indexing
tmpxyz=augmentedStrandSet.strandData(:,2:4)+1;
% get singly-unconnected endpoints from connectionsretest_jk and connections_jk
if VesselRepresentations.GapConnectionParameters.PreprocessNNConnections    % If the small obvious gaps should be connected via NN, set this to 1
    singleuconepts=unique([connectionsretest_jk.pt2pt]);                    % WARNING: NN connections do not produce bridging strands
else
    singleuconepts=unique([[connectionsretest_jk.pt2pt],[connections_jk.pt2pt]]);
end
%% Preallocate
connectionsbyrelaxation=struct([]);                          % output structure
BridgeStrands=cell(1,numel(singleuconepts));                 % cell array of strand centerline points
GapConfidences = zeros(1,numel(singleuconepts));             % confidence in each connection
% toggle display
dispon = 0;

tmpresultsfname = ['partialresults-',datestr(clock,30),'.mat'];

for iCONN = 1:numel(singleuconepts)
    disp(['Analyzing candidate point number: ',num2str(iCONN),' of ',num2str(numel(singleuconepts)),'...']);
    ptconn1=singleuconepts(iCONN);
    strand1=(augmentedStrandSet.ctrPoint2StrandLUT(ptconn1,2));
    s1inds=strandSet.ctrPoint2StrandLUT(:,2)==strand1;
    % compute the bounding box that will ~minimally contain the strands associated with endpoints
    [vollimits] = round(getstrandBB(tmpxyz(ptconn1,:),VesselRepresentations.GapConnectionParameters.BBmargin));
    [vollimits] = boundscheck(vollimits,[1 1 1,Vvecsz(2),Vvecsz(1),Vvecsz(3)]);
    [x y z] = meshgrid(vollimits(1):vollimits(2), vollimits(3):vollimits(4), vollimits(5):vollimits(6));
    % set coords based on vollimits
    coords.x1min=vollimits(1);coords.x1max=vollimits(2);coords.x2min=vollimits(3);coords.x2max=vollimits(4);coords.x3min=vollimits(5);coords.x3max=vollimits(6);
    % get the local subvolume to render (B=binary mask, R=raw volume, D=enhanced volume)
    B = GetBlockFromStack(VesselRepresentations.IsotropicVesselMaskMatFname,coords);
    B = logical(permute(B,[2,1,3]));
    R = GetBlockFromStack(VesselRepresentations.RawVesselVolumeMatFname,coords);
    R = single(permute(R,[2,1,3]));
    D = GetBlockFromStack(VesselRepresentations.EnhancedVesselVolumeMatFname,coords);
    D = single(permute(D,[2,1,3]));

    if dispon
        % Draw binary mask in bounding box with strands
        figure;clf;hold on;
        patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
        atmp = augmentedStrandSet;
        inbb1 = (vollimits(3)<= atmp.strandData(:,2)) & (vollimits(4)>= atmp.strandData(:,2));
        inbb2 = (vollimits(1)<= atmp.strandData(:,3)) & (vollimits(2)>= atmp.strandData(:,3));
        inbb3 = (vollimits(5)<= atmp.strandData(:,4)) & (vollimits(6)>= atmp.strandData(:,4));
        inbb = inbb1 & inbb2 & inbb3;
        atmp.strandData(~inbb,:) = NaN;
        jk_amira2mat_displayLabeledStrands(atmp);
        grid on;axis equal;
        axis(vollimits)
        axis off;
    end
    % Get intensity distribution of vessel along centerline points on strands associated with endpoints
    disp('Getting local intensity distribution');
    [LocalDist,Thresholds,FineThresholds] = GetLocalIntensityDistribution(D,R,B);
    % Get disconnected endpoint strand masks (mask only on strands connected to endpoints)
    disp('Getting endpoint and strand masks');
    [EPStrandMask,EPMask] = GetSingleEndpointStrandMask(B,x,y,z,tmpxyz(s1inds,:),tmpxyz(ptconn1,:));
    % Choose a new mask that connects the current endpoint to some other centerline point, return the candidate point
    % that connects ptconn1 (in ptconn2) and the endpoint location found (in ept2)
    disp('Finding connecting point via threshold relaxation...');
    [ptconn2,ept2,BacktrackMask] = ConnectedCenterlinePointThreshRelax(D,B,EPStrandMask,EPMask,tmpxyz(ptconn1,:),tmpxyz,FineThresholds,x,y,z,VesselRepresentations.GapConnectionParameters.threshparameters.minvvol,VesselRepresentations.GapConnectionParameters.threshparameters.morphfiltsz);

    if ~isempty(ptconn2) && ~isempty(ept2)
        strand2=(augmentedStrandSet.ctrPoint2StrandLUT(ptconn2,2));
        s2inds=strandSet.ctrPoint2StrandLUT(:,2)==strand2;
        disp('Finding the gap connecting mask...');
        [GapConnectingMask,GapConfidence] = ChooseGapConnectingMask(D,R,B,BacktrackMask,EPMask,tmpxyz(ptconn1,:),tmpxyz(ptconn2,:),Thresholds,x,y,z,VesselRepresentations.GapConnectionParameters.threshparameters.minvvol,VesselRepresentations.GapConnectionParameters.threshparameters.morphfiltsz);

        fprintf(1,'Gap confidence was %2.2f\n',GapConfidence);
        if ~isreal(GapConfidence)
            disp('gap confidence is imaginary');
            keyboard;
        end
        
        IncompleteBridgingStrandComputation = false;
        try
            % Find the minimal path from EP1 to EP2 in GapConnectingMask
            disp('Estimating the bridging strand...');
            [BridgeStrandEP1EP2] = ShortestBridgeEP1ToEP2(tmpxyz(ptconn1,:),tmpxyz(ptconn2,:),GapConnectingMask,x,y,z);
            % Save the strand and confidence in a cell array
            BridgeStrands{iCONN}=BridgeStrandEP1EP2;
            GapConfidences(iCONN)=GapConfidence;
        catch
            % if no connection, save empty strands and 0 confidences in a cell and structure array
            BridgeStrands{iCONN} = [];
            GapConfidences(iCONN) = 0;
            IncompleteBridgingStrandComputation = true;
        end
    else
        % if no connection, save empty strands and confidences in a cell and structure array
        BridgeStrands{iCONN} = [];
        GapConfidences(iCONN) = 0;
    end
    
    % surface render binary mask near endpoint connection
    if dispon
        % Draw binary mask in bounding box with strands with gap connecting mask and bridging strand
        h=figure;clf;hold on;
        patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
        % draw strands associated with endpoints with solid black
        plot3(tmpxyz(s1inds,2),tmpxyz(s1inds,1),tmpxyz(s1inds,3),'k.-','linewidth',2,'markersize',15);
        % draw endpoints with solid black circles
        plot3(tmpxyz(ptconn1,2),tmpxyz(ptconn1,1),tmpxyz(ptconn1,3),'o','color',[.4 .4 0],'markersize',10);
        if ~isempty(ptconn2) && ~isempty(ept2)
            % draw bridge strand with solid black line
            plot3(tmpxyz(ptconn2,2),tmpxyz(ptconn2,1),tmpxyz(ptconn2,3),'ko','markersize',10);
            patch(isosurface(x,y,z,GapConnectingMask, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
            plot3(tmpxyz(s2inds,2),tmpxyz(s2inds,1),tmpxyz(s2inds,3),'k.-','linewidth',2,'markersize',15);
            plot3(BridgeStrandEP1EP2(:,1),BridgeStrandEP1EP2(:,2),BridgeStrandEP1EP2(:,3),'m','linewidth',3);
            plot3(ept2(2),ept2(1),ept2(3),'k.','markersize',10);
        end
        grid on;axis equal
        view(30,30)
        set(h,'position',[520   100   560   820])
    end
    pause(1);
    
    close all;
    if (~isempty(ptconn2) && ~isempty(ept2)) && ~IncompleteBridgingStrandComputation
        connectionsbyrelaxation(iCONN).pt2pt = [ptconn1,ptconn2];
    else
        connectionsbyrelaxation(iCONN).pt2pt = [ptconn1,ptconn1];
    end

    disp('saving temporary results...');
    save(tmpresultsfname);

    disp('check tmp2.mat if error stopped processing');
end
save tmp3
clear R D B BacktrackMask EPMask EPStrandMask GapConnectingMask x y z
% Get strand length and pt-to-pt distance statistics
BridgeStrandStepLengths = zeros(1,numel(BridgeStrands));
BridgeStrandDists = zeros(1,numel(BridgeStrands));
for iBRS = 1:numel(BridgeStrands)
    if ~isempty(BridgeStrands{iBRS})
        BridgeStrandStepLengths(iBRS) = sum(sum(abs(diff(BridgeStrands{iBRS}))));
        BridgeStrandDists(iBRS) = sqrt(sum((BridgeStrands{iBRS}(1,:)-BridgeStrands{iBRS}(end,:)).^2));
    end
end
%% COLLECT AND ORGANIZE OUTPUT STRUCTURE DESCRIBING STRANDSET AND CONNECTIONS FOUND BY ALGORITHM
% Populate the returned strandset
GapConnectionStruct.strandSet = augmentedStrandSet;

if VesselRepresentations.GapConnectionParameters.PreprocessNNConnections
    % Populate the pairs of connected endpoints
    GapConnectionStruct.allconnectionptpairs = [reshape([connections_jk.pt2pt],2,numel(connections_jk))';reshape([connectionsbyrelaxation.pt2pt],2,numel(connectionsbyrelaxation))'];
    % Populate the pairs of connected endpoints
    GapConnectionStruct.connectiontypes = [repmat('NN',numel(connections_jk),1);repmat('TR',numel(connectionsbyrelaxation),1)];
else
    % Populate the pairs of connected endpoints
    GapConnectionStruct.allconnectionptpairs = reshape([connectionsbyrelaxation.pt2pt],2,numel(connectionsbyrelaxation))';
    % Populate the pairs of connected endpoints
    GapConnectionStruct.connectiontypes = repmat('TR',numel(connectionsbyrelaxation),1);
end

% Populate the bridging strand endpoints to add to the .mv3d file
GapConnectionStruct.ThreshRelax.BridgingStrands = BridgeStrands;
GapConnectionStruct.ThreshRelax.BridgingConfidences = GapConfidences;
