function strandSet = strandify(CTP)
%STRANDIFY - creates strands from center points
%
%Inputs CTP is a structure with the following fields
%   Adj - center point adjacency matrix
%   x,y and [z] fields for the coordinates of each center point
%   [rad] - radius associated with center point
%
%Output
%   strandSet structure suitable for gap binding algorithm
%
% Created by Pablo Blinder
clc
strandSet = [];
%ensure matrix is symetric
% CTP.Adj = adj2sym(CTP.Adj);
CTP.Adj = abs(CTP.Adj + CTP.Adj')>0;
CTPdeg = full(sum(CTP.Adj));
CTPtoAssign = find(CTPdeg<3);
strandIni = find(CTPdeg >=3);
nStrandIni = numel(strandIni);

%keep also list of branch to branch direct connections
ini2ini = [];

%cycle strand initialization points
validStrands = 0;
for ni = 1 : nStrandIni
    %create queue  with set of adjacent vertices to ini
    thisStrandIniQueue = find(CTP.Adj(strandIni(ni),:));
    thisStrandIniQueueAreIni = thisStrandIniQueue(CTPdeg(thisStrandIniQueue) >= 3);
    %process prior to moving on
    for ki = 1 : numel(thisStrandIniQueueAreIni)
        %check if current connection already exists
        this = sort([strandIni(ni) thisStrandIniQueueAreIni(ki)]);
        nrows = size(ini2ini,1);
        alreadyConnected = sum(ismember(repmat(this,nrows,1),ini2ini,'rows'));
        if ~alreadyConnected
            validStrands = validStrands + 1;
            strandSet(validStrands).ptr2CTP = this';
            ini2ini = [ini2ini ; this];
        end
    end %processing ini2ini
    
    
    %make sure each ctp that is not a strandIni gets assigened only once
    thisStrandIniQueue = intersect(thisStrandIniQueue,CTPtoAssign);
    queuePtr = 1;
    while ~isempty(thisStrandIniQueue) && queuePtr <= numel(thisStrandIniQueue)
        %initialize each strand witht the strand head and the current
        %element of the queue 
        currentCTP = thisStrandIniQueue(queuePtr);
        strandCTPlist = [strandIni(ni) ; currentCTP ];
%         strandPtr = 2;


        %add ctp as long as their vertex degree is less than 3
        %terminate strand if vertex degreee is either 1 or higher than 3
        %get adjacent vertices to current element in que

        keepAdding = 1;
        while keepAdding
            candidateCTP = setdiff(find(CTP.Adj(currentCTP,:)),strandCTPlist);
            if ~isempty(candidateCTP)
                %analyze candidte ctp(s)
                candidateCTPdeg = CTPdeg(candidateCTP);
                if candidateCTPdeg == 1 || candidateCTPdeg >= 3
                    %add to strand and terminate
                    strandCTPlist = [strandCTPlist;candidateCTP];
                    keepAdding = 0; %terminate
                else
                    strandCTPlist = [strandCTPlist ; candidateCTP];
                    currentCTP = candidateCTP;
                end %analyzing candidate ctp

            else
                keepAdding  = 0;
            end %non empty set of candidate CTP
        end %adding ctps to current strand

        %add current strand to strand set
        validStrands = validStrands + 1;
        strandSet(validStrands).ptr2CTP = strandCTPlist;
        %remove from ctp list to assign
        CTPtoAssign = CTPtoAssign(~ismember(CTPtoAssign,strandCTPlist));

        queuePtr = queuePtr + 1; %push que counter
    end% processing queue
end

%% Simple strand plotter
map = prism(validStrands);
figure('color','w','windowStyle','docked');hold on;
h2Lines = [];
for si = 1 : validStrands
    CTPidx = strandSet(si).ptr2CTP;
    h2Lines(si) = plot(CTP.x(CTPidx),CTP.y(CTPidx),'UserData',si);
end
Pn(1)={'Color'};
Pv=num2cell(map,2);
set(h2Lines,'LineWidth',2.5,...
    Pn,Pv,'ButtonDownFcn','get(gco,''UserData'')');
hold on
gplot(CTP.Adj,[CTP.x' CTP.y'],'k.');

%% generate strand set (as standarized for Amira2mat and downstream gap binding algoritm
if ~isfield(CTP,'z')
    CTP.z = ones(numel(CTP.x),1);
end
nCTP = numel(vertcat(strandSet.ptr2CTP));
dataInFile = zeros(nCTP,5);
CTP.rad = ones(nCTP,1); %%%%% DEV ONLY %%%%% 
disp('WARNING: no radii data, all set to 1')
ptr = 1;
for si = 1 : validStrands
    CTPidx = strandSet(si).ptr2CTP;
    chunkSize = numel(CTPidx);
    chunk = [repmat(si,chunkSize,1) CTP.x(CTPidx)' CTP.y(CTPidx)' CTP.z(CTPidx)' CTP.rad(CTPidx)];
    dataInFile(ptr:ptr + chunkSize - 1,:) = chunk;
    ptr  = ptr + chunkSize;
end

t0 = clock;
tic;
idx2StrandsInFile = [0 ;  find(diff(dataInFile(:,1))) ; size(dataInFile,1)];
nStrands = numel(idx2StrandsInFile) - 1;
fprintf('*** Parsing  ');
strandSetInfo = struct('lineId',[],'ptr2coord',[]);
strandSetInfo(nStrands).lineId = nStrands;
LUT = [];
for ki = 1 : nStrands
    strandSetInfo(ki).lineId = ki;
    strandSetInfo(ki).ptr2coord = (idx2StrandsInFile(ki) + 1 : idx2StrandsInFile(ki + 1))';
    LUT = [LUT;...
        [strandSetInfo(ki).ptr2coord repmat(ki,numel(strandSetInfo(ki).ptr2coord),1)]];
end % cycling line indices

% Build strandSet structure
strandSet = struct('strandData',dataInFile,'strandSetInfo',...
    strandSetInfo,'ptr2endpoints',idx2StrandsInFile,'ctrPoint2StrandLUT',LUT);

fprintf(' %d strands took %f sec \n',nStrands,etime(clock,t0))
    