function varargout = VidaCorrelateStacks(inputParameters)
%
%This program takes two z-stack tifs that overlap slightly in x or y
%and determines the optimal yxz position of the second stack relative to
%the first.
%
%Input arguments are stored in the structure inputParameters as:
%   inputParameters.inputFileName1 =  path & filename string of a upper/left file
%   inputParameters.inputFileName2 =  path & filename string of a lower/right file
%   inputParameters.subVolumeYXZLimits =  
%   inputParameters.searchRangeYXZInVoxels = x,y,z search sizes (+/-)
%   inputParameters.filterSizeYXZInVoxels = low pass filter kernel for image before correlation
%   inputParameters.projectionMethod = choice between mean projection or max projection
%   inputParameters.yxzGuessInVoxels = best guess yxz_offsets between the two stacks
%
%Output argument is the 1x3 matrix containing the optimal x,y,and z positions for
%the second (lower) stack relative to the first.  Remember the yx order swtich when
%loading tiff files!
%   inputParameters.best_yxz = [bestx,besty,bestz]

%USE:
%BEST_YXZ = CORRELATE_YSTRIP(MYPARAMETERS);
%
%Sample Script:
% inputParameters.inputFileName1 =  'C:\lj01-Z1\lj01-Z1-C1-R1-Ch3b.tif'
% inputParameters.inputFileName1 =  'C:\lj01-Z1\lj01-Z1-C1-R2-Ch3b.tif'
% inputParameters.subVolumeYXZLimits = [16 35 51 200 21 100];
% inputParameters.searchRangeYXZInVoxels = [10 10 10];
% inputParameters.filterSizeYXZInVoxels = [5,5,5];
% inputParameters.projectionMethod = 'max'; 
        %or 'mean'
% inputParameters.yxzGuessInVoxels = [-4 196 -1];
% best_yxz = correlateStacks(inputParameters);
%
%
% Written by Phil Tsai 08/10/06
% Last Updated 10/18/06


inputFileName1 = inputParameters.inputFileName1;
inputFileName2 = inputParameters.inputFileName2;
subVolumeYXZLimits = inputParameters.subVolumeYXZLimits;
searchRangeYXZInVoxels = inputParameters.searchRangeYXZInVoxels;
filterSizeYXZInVoxels = inputParameters.filterSizeYXZInVoxels;
projectionMethod = inputParameters.projectionMethod;
yxzGuessInVoxels = inputParameters.yxzGuessInVoxels;

%Get Block Size


%some variables for ease of coding
y1 = subVolumeYXZLimits(1); y2 = subVolumeYXZLimits(2);
x1 = subVolumeYXZLimits(3); x2 = subVolumeYXZLimits(4);
z1 = subVolumeYXZLimits(5); z2 = subVolumeYXZLimits(6);
xplus = x2-x1; yplus = y2-y1; zplus = z2-z1;
yrange = searchRangeYXZInVoxels(1); yshift = [-yrange:yrange];
xrange = searchRangeYXZInVoxels(2); xshift = [-xrange:xrange];
zrange = searchRangeYXZInVoxels(3); zshift = [-zrange:zrange];
%yxfilt = ones(filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(1));
%xzfilt = ones(filterSizeYXZInVoxels(1),filterSizeYXZInVoxels(3));
%yzfilt = ones(filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(3));
yxfilt = gaussian2d(2*filterSizeYXZInVoxels(1),filterSizeYXZInVoxels(1),2*filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(2));
xzfilt = gaussian2d(2*filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(1),2*filterSizeYXZInVoxels(3),filterSizeYXZInVoxels(3));
yzfilt = gaussian2d(2*filterSizeYXZInVoxels(1),filterSizeYXZInVoxels(2),2*filterSizeYXZInVoxels(3),filterSizeYXZInVoxels(3));
yxzfilt = gaussian3d(2*filterSizeYXZInVoxels(1),filterSizeYXZInVoxels(1),2*filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(2),filterSizeYXZInVoxels(3)*2,filterSizeYXZInVoxels(3));
yskip = yxzGuessInVoxels(1); xskip = yxzGuessInVoxels(2); zskip = yxzGuessInVoxels(3);



%load/filter/project/crop the second stack (the interrogating stack)
mypics2 = loadmovie(inputFileName2,[0,0,0,0,z1-3,z2+3]);
tempsub2 = double(mypics2(y1-3:y2+3,x1-3:x2+3,:));%crop out alittle around the analysis region for quick filtering

%make the yx projection for stack1
switch projectionMethod
    case {'Max','MAX','max'}
        tempyx = squeeze(max(tempsub2,[],3));
    case {'Mean','mean','MEAN'}
        tempyx = squeeze(mean(tempsub2,3));
    otherwise
        tempyx = squeeze(mean(tempsub2,3));
end%switch projectionMethod

overyx = imfilter(tempyx,yxfilt,'replicate');
yxproj2 = overyx(4:4+yplus,4:4+xplus); %crop out just the interrogating region
yxproj2b = yxproj2-mean(mean(yxproj2));
yxdiv2 = sqrt(sum(sum(yxproj2b.^2)));

if xplus > yplus,
    %make the xz projection for stack1
    switch projectionMethod
        case {'Max','MAX','max'}
            tempxz = squeeze(max(tempsub2,[],1));
        case {'Mean','mean','MEAN'}
            tempxz = squeeze(mean(tempsub2,1));
        otherwise
            tempxz = squeeze(mean(tempsub2,1));
    end%switch projectionMethod
    overxz = imfilter(tempxz,xzfilt,'replicate');
    xzproj2 = overxz(4:4+xplus,4:4+zplus); %crop out just the xz interrogating region
    xzproj2b = xzproj2-mean(mean(xzproj2));
    xzdiv2 = sqrt(sum(sum(xzproj2b.^2)));
else %if xplus > yplus
    %make the yz projection for stack1
    switch projectionMethod
        case {'Max','MAX','max'}
             tempyz = squeeze(max(tempsub2,[],2));
        case {'Mean','mean','MEAN'}
            tempyz = squeeze(mean(tempsub2,2));
        otherwise
            tempyz = squeeze(mean(tempsub2,2));
    end%switch projectionMethod
    overyz = imfilter(tempyz,yzfilt,'replicate');
    yzproj2 = overyz(4:4+yplus,4:4+zplus); %crop out just the yz interrogating region
    yzproj2b = yzproj2-mean(mean(yzproj2));
    yzdiv2 = sqrt(sum(sum(yzproj2b.^2)));
end %if xplus > yplus


%find the coordinates for the interrogated window in stack1
xp1 = x1+xskip-xrange; xp2 = xp1+xplus+2*xrange+1;
yp1 = y1+yskip-yrange; yp2 = yp1+yplus+2*yrange+1;
zp1 = z1+zskip-zrange; zp2 = zp1+zplus+2*zrange+1;

%load/project/filter the first stack (the interrogated stack)
mypics1 = loadmovie(inputFileName1,[0,0,0,0,zp1-3,zp2+3]);

try
    tempsub1 = double(mypics1(yp1-3:yp2+3,xp1-3:xp2+3,:));
catch
    disp('Out of Range Error While Correlating Stacks for Tiling.');
    disp('   TilingParameters.marginFraction may be too small or searchRange too large');
    keyboard
end
tempnewpics1 = imfilter(tempsub1,yxzfilt,'replicate');
newpics1 = tempnewpics1(4:4+yplus+2*yrange,4:4+xplus+2*xrange,4:4+zplus+2*zrange);


%example calculation
% xskip = 200; xrange = 10; x1 = 15; x2 = 90 =>
% xplus = 75; xp1 = 205 ; xp2 = 301;
% 
% when x1 = 15, xx1 should be 215, xx2 should be 290
% newpics runs from 205 to 301
% so 205 is actual 1, 215 is actually 11
% and 290 is actually 86
% so X1 = kx 

%crop a sample of the projected interrogated stack and yx cross-correlate

Z1 = zrange; Z2 = zrange+zplus;
mycorryx = zeros(length(xshift),length(yshift));
for kx = 1:length(xshift);
    X1 = kx;
    X2 = kx+xplus;
    for ky = 1:length(yshift);
        Y1 = ky;
        Y2 = ky+yplus;
        mysub1 = newpics1(Y1:Y2,X1:X2,Z1:Z2);
        switch projectionMethod
            case {'Max','MAX','max'}
                yxproj1 = squeeze(max(mysub1,[],3));
            case {'Mean','mean','MEAN'}
                yxproj1 = squeeze(mean(mysub1,3));
            otherwise
                yxproj1 = squeeze(mean(mysub1,3));
        end%switch projectionMethod
        yxproj1b = yxproj1 - mean(mean(yxproj1));
        tempcorr = sum(sum(yxproj1b.*yxproj2b));
        yxdiv1 = sqrt(sum(sum(yxproj1b.^2)));
        mycorryx(ky,kx) = tempcorr./(yxdiv1.*yxdiv2);       
        %figure(1);imagesc(yxproj1b);
        %figure(2);imagesc(yxproj2b);
        %pause;
    end
end

maxcorryx = max(max(mycorryx));
wheremaxyx = find(mycorryx == maxcorryx);
[ym,xm] = ind2sub(size(mycorryx),wheremaxyx);


%now find the z-correlation
mycorrz = zeros(length(zshift));
X1 = xm;
X2 = xm+xplus;
Y1 = ym;
Y2 = ym+yplus;
for kz = 1:length(zshift);
    Z1 = kz;
    Z2 = kz+zplus;
        mysub1 = newpics1(Y1:Y2,X1:X2,Z1:Z2);
        if xplus > yplus, %perform an xz projection correlation
            switch projectionMethod
                case {'Max','MAX','max'}
                    xzproj1 = squeeze(max(mysub1,[],1));
                case {'Mean','mean','MEAN'}
                    xzproj1 = squeeze(mean(mysub1,1));
                otherwise
                    xzproj1 = squeeze(mean(mysub1,1));
            end%switch projectionMethod
            xzproj1b = xzproj1 - mean(mean(xzproj1));
            tempcorr = sum(sum(xzproj1b.*xzproj2b));
            xzdiv1 = sqrt(sum(sum(xzproj1b.^2)));
            mycorrz(kz) = tempcorr./(xzdiv1.*xzdiv2);
%             figure(1);imagesc(xzproj1b);
%             figure(2);imagesc(xzproj2b);
%             pause;
%             kz
        else %if xplus > yplus  ; do an yx projection correlation instead
           switch projectionMethod
                case {'Max','MAX','max'}
                    yzproj1 = squeeze(max(mysub1,[],2));
                case {'Mean','mean','MEAN'}
                    yzproj1 = squeeze(mean(mysub1,2));
                otherwise
                    yzproj1 = squeeze(mean(mysub1,2));
            end%switch projectionMethod
            yzproj1b = yzproj1 - mean(mean(yzproj1));
            tempcorr = sum(sum(yzproj1b.*yzproj2b));
            yzdiv1 = sqrt(sum(sum(yzproj1b.^2)));
            mycorrz(kz) = tempcorr./(yzdiv1.*yzdiv2);
%             figure(1);imagesc(yzproj1b);
%             figure(2);imagesc(yzproj2b);
%             pause;
%             kz
        end %if xplus > yplus
end

maxcorrz = max(mycorrz);
wheremaxz = find(mycorrz == maxcorrz(1));
zm = ind2sub(size(mycorrz),wheremaxz);

besty = ym - (yrange + 1) + yskip;
bestx = xm - (xrange + 1) + xskip;
bestz = zm - (zrange + 1) + zskip;

varargout{1} = [besty,bestx,bestz];
return%function



