function outputParameters = TileImages(inputParameters);
%
%This program makes a 3D montage .tif file from individual 3D tiffs.
%  It holds the entire final montage in memory and so is limited to medium-sized montages
%  It requires the position of each stack in the global stack as output by
%  xymontage_locate.m (which in turn uses correlateStacks.m)
%  No overlap averaging is performed.  Around the perimeter of the montage,
%       a more extended field extent is used
%
%Input arguments are stored in the structure inputParameters as:
%   inputParameters.sampleFileName = any one of the -Cx-Ry-Chm.tif style file names;
%   inputParameters.numXFields = number of rightward tiling stacks;
%   inputParameters.numYFields = number of downward tiling stacks;
%   inputParameters.reverseXDirectionFlag = 0 (no) / 1 (yes) : reverse column file #s if
%           data originally taken in rightward / leftward direction
%   inputParameters.reverseYDirectionFlag = 0 (no) / 1 (yes) : reverse row file #s if
%           data originally taken in rightward / leftward direction
%   inputParameters.finalYXZ = cell containing the [xyz] coords of each stack
%   inputParameters.finalYXZSize = [x,y,z] size of required final montage
%   inputParameters.finalX = [stackX,stackY] matrix of x positions
%   inputParameters.finalY = [stackX,stackY] matrix of y positions
%   inputParameters.finalZ = [stackX,stackY] matrix of z positions
%   inputParameters.xCropStart = left-side cropping before placing
%   inputParameters.yCropStart = top-side cropping before placing
%   inputParameters.zCropStart = upper-z-side cropping before placing
%   inputParameters.outputFileName = desired outputFileName
%   inputParameters.waitbarFlag = yes/no show waitbar;
%
%Output argument is the (modified) structure inputParameters
%If no input variable is given, a user input window displays.
%If the first input argument is not a structure, it is assumed to be a
%the inputFileName string
%
%USE:
%OUTPUT_PARAMETERS = XYMONTAGE(MYPARAMETERS);
%
%Sample Script
%
%
% Written by Phil Tsai 08/10/06
% Last Updated 08/31/06

%-----
processedDataFolder = inputParameters.processedDataFolder;
fileRootName = inputParameters.fileRootName;
generateOutputTifsFlag = inputParameters.generateOutputTifsFlag;
%-----
sampleFileName = inputParameters.sampleFileName;
relativeYXZ = inputParameters.relativeYXZ;
currentFullInputFileRoot = inputParameters.currentFullInputFileRoot;
outputFileName = inputParameters.outputFileName;
numXFields =  inputParameters.numXFields;
numYFields =  inputParameters.numYFields;
subVolumeTileHorizontalOverlapFraction = inputParameters.subVolumeTileHorizontalOverlapFraction;
subVolumeTileVerticalOverlapFraction = inputParameters.subVolumeTileVerticalOverlapFraction;
reverseXDirectionFlag =  inputParameters.reverseXDirectionFlag;
reverseYDirectionFlag =  inputParameters.reverseYDirectionFlag;
%-----


%Obtain image size information
myinfo = imfinfo(sampleFileName);
zSizeTrue = length(myinfo);
xSizeTrue = myinfo(1).Width;
ySizeTrue = myinfo(1).Height;
%-----
overlapSizeX = round(subVolumeTileHorizontalOverlapFraction * xSizeTrue);
overlapSizeY = round(subVolumeTileVerticalOverlapFraction * ySizeTrue);
yCropStart = ceil(overlapSizeY/2);
xCropStart = ceil(overlapSizeX/2);


pairIter = 0;
%Calculate the absolute position
for xIter = 1:numXFields
    for yIter = 1:numYFields
        pairIter = pairIter + 1;
        r = relativeYXZ{pairIter};
        if yIter == 1, %top of column
            if xIter == 1, %very first stack
                abspos{yIter,xIter} = [0 0 0];
            else%if xIter == 1, 
                abspos{yIter,xIter} = abspos{yIter,xIter-1} + r;
            end%if xIter==1,
        else%if yIter==1,
            abspos{yIter,xIter} = abspos{yIter-1,xIter} + r;
        end%if-else yIter==1,
        %---
        temp = abspos{yIter,xIter};
        ypos(yIter,xIter) = temp(1);
        xpos(yIter,xIter) = temp(2);
        zpos(yIter,xIter) = temp(3);
    end%for yIter=1:numy
end%for xIter=1:numx
%-----
%Now calculate required size of montage and final coordinates of each stack
minx = abs(min(min(xpos)))+1;
miny = abs(min(min(ypos)))+1;
minz = abs(min(min(zpos)))+1;
for xIter = 1:numXFields
    for yIter = 1:numYFields
        finalYXZ{yIter,xIter} = abspos{yIter,xIter} + [miny, minx, minz];
        temp = finalYXZ{yIter,xIter};
        finalY(yIter,xIter) = temp(1);
        finalX(yIter,xIter) = temp(2);
        finalZ(yIter,xIter) = temp(3);
    end
end
%Calculate necessary size for final montage
myinfo = imfinfo(sampleFileName);
zSize = length(myinfo);
xSize = myinfo(1).Width;
ySize = myinfo(1).Height;
%---
finalYSize = ySize + max(max(finalY))-1;
finalXSize = xSize + max(max(finalX))-1;
finalZSize = zSize + max(max(finalZ))-1;
finalYXZSize = [finalYSize,finalXSize,finalZSize];
%---





yplus = ySizeTrue - yCropStart;
xplus = xSizeTrue - xCropStart;
zplus = zSizeTrue - 1;


%determine order of files to open;
colNum = [1:numXFields];
rowNum = [1:numYFields];
if reverseXDirectionFlag == 1, colNum = [numXFields:-1:1]; end
if reverseYDirectionFlag == 1, rowNum = [numYFields:-1:1]; end




mybit = uint16(0);
imageStack = repmat(mybit,[finalYXZSize(1),finalYXZSize(2),finalYXZSize(3)]);


%Begin Tiling Frames
numStacks = numYFields*numXFields;
zs = 1; zp = zplus;
for kx = 1:numXFields;
    if kx == 1; xs = 1; xp = xSizeTrue-1 ; else xs = xCropStart; xp = xplus; end
    for ky = 1:numYFields,
        currentStack = (numYFields*(kx-1))+ky;
        StatusMessage = ['Tiling ',num2str(currentStack),' of ',num2str(numStacks)];
        charCount = fprintf(StatusMessage);
        if ky == 1; ys = 1; yp = ySizeTrue-1 ; else ys = yCropStart; yp = yplus; end
        startX = finalX(ky,kx)+xs-1; startY = finalY(ky,kx)+ys-1; startZ = finalZ(ky,kx)+zs-1;
        zp = zplus;
        currentFileName = [currentFullInputFileRoot,'-Z01-C',num2str(colNum(kx),'%02d'),'-R',num2str(rowNum(ky),'%02d'),'.tif'];
        allFileNames{ky,kx} = currentFileName;
        temppic = loadmovie(currentFileName,'','','','','','','no');
        imageStack(startY:startY+yp,startX:startX+xp,startZ:startZ+zp) = temppic(ys:ys+yp,xs:xs+xp,zs:zs+zp);
        fprintf(repmat('\b',[1,charCount]));
    end
end

% % if compensateZDriftFlag == 1,
% %     %Correct for zShifting if requested
% %     zShiftInY =  floor(mean(finalZ(:,1)-finalZ(:,5)));
% %     zShiftInX =  floor(mean(finalZ(1,:)-finalZ(5,:)));
% %     yOver = floor(mean(finalY(:,1)-finalY(:,5)));
% %     xOver = floor(mean(finalX(1,:)-finalX(5,:)));
% %     zShiftPerY = zShiftInY / yOver;
% %     zShiftPerX = zShiftInX / xOver;
% %     zShiftInY = -1*floor(cumsum(ones([1,finalYXZSize(1)]).*zShiftPerY));
% %     zShiftInX = -1*floor(cumsum(ones([1,finalYXZSize(2)]).*zShiftPerX));
% %     %---
% %     for kx = 1:finalYXZSize(1);
% %         temppic = squeeze(imageStack(:,kx,:));
% %         temppic2 = circshift(temppic,[0,zShiftInX(kx)]);
% %         imageStack(:,kx,:) = temppic2;
% %     end
% %     %---
% %     for ky = 1:finalYXZSize(2);
% %         temppic = squeeze(imageStack(ky,:,:));
% %         temppic2 = circshift(temppic,[0,zShiftInY(ky)]);
% %         imageStack(ky,:,:) = temppic2;
% %     end
% % end%if compensateZDriftFlag == 1,

%-----
imageStackSize = size(imageStack);
saveSize = (getfield(whos('imageStack'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(outputFileName,'imageStack',scs);
%-----
if generateOutputTifsFlag == 1,
    outputTifFileName = strrep(outputFileName,'.mat','.tif');
    maketiff(imageStack,outputTifFileName);
end%if generateOutputTifsFlag == 1,
%-----
outputParameters = inputParameters;
outputParameters.outputFileName = outputFileName;
outputParameters.imageStackSize = imageStackSize;
%-----
return


            
            