


%Start from each base strand - one at a time
%The # of particles is proportional to the current in that base strand
%The particles split at each junction if (1 mother -> 2 daughters)
%If junction is (2 mothers -> 1 daughter), no split, but no addition.
%At each strand, keep a running list of:
%       [# of particles added, transit time of that set of particles]
%Transit time = Transit time + length(strand) / current(strand)

%Repeat for every base strand

%At each strand take the running list and calculated the weighted-average
%   transit time

%Assumptions made:
%   1) for all vessels, current = velocity; true for single file vessels
%       but for diameters > ~ 12um, may need to implement: 
%       velocity = current * correctionFactor
%           correctionFactor = (vesselDiameter/RBCDiameter)^2 ; hard minimum of 1
%   2) oxygen depletion is going to be assumed to be linear with transit time
%       ignores exponential slow down of depletion as gradient (blood:tissue) decreases
%       assumes diffusion across vessel tissue same for all vessels
%       sanityCheck: MCA diameter (mouse) ~150um, MCA velocity ~500 mm/s
%           Assume blood travels ~50 mm (2inch) from heart to plunging arteriole in this size vessel.
%           => 0.1 second transit.
%           Now assume flow in capillaries is ~ 0.5 mm/s, 1 mm of vessels:
%           surface-bed-surface 2 seconds;
%           Therefore 20x as long in capillary as in artery
%           => ~5% depletion in artery compared to capillaries : Consistent.
global gds

minCurrentFractionLimit = 0.001;

Strands = gds.Strands;
strandCurrentList = gds.clotAnalysisParameters.strandCurrentList;
strandConductanceList = gds.clotAnalysisParameters.strandConductanceList;
AllVoltages = gds.Vertices.AllVoltages;
StrandIndices = gds.Vertices.StrandIndices;
baseStrandIDList = gds.baseStrandIDList;
%---
numStrands = numel(strandConductanceList);
numBaseStrands = numel(baseStrandIDList);



%Generate a transitTimeList
ttMultiplier = 1;
transitTimeList = zeros(numStrands,1);
for strandIter = 1:numStrands,
    thisStrand = Strands(strandIter);
    thisTransitTime = ttMultiplier * (numel(thisStrand.InteriorVertices)+1)  / abs(thisStrand.strandCurrent);
    transitTimeList(strandIter) = thisTransitTime;
end



%Start from a base strand: give that strand a particle count(pc) = its current
%   and a total transit time (ttt) of 0
%These strands act at seed strands
%For each seed strand:
%   Find lower voltage node attached
%       Find the other strands attached to the lower node
%       These strands get a ttt = ttt(seedStrand) + seedStrand's singleTransitTime
%       The sum(pc(all daughters)) = pc(mother), split by currentRatio(daughters) 
%       Mark these strands as next round seeds
%
%Next round: find those strands marked as seeds.
%   Repeat. If a ttt+pc is already attached to a strand, append.

%When all paths have gone to completion (no lower voltage nodes available)
%   Start with the next base strand and repeat the whole
%   procedure.

%%
%Pick a seed strand

ttt = zeros(numStrands,numBaseStrands)+inf; %ttt = total transit time
pc = zeros(numStrands,numBaseStrands); %pc = particle count;


for baseStrandIter = 1:numBaseStrands;
    
    %figure out which attached node point is the lower voltage;
    seedStrandIDList = baseStrandIDList(baseStrandIter);
    thisSeedStrandID = seedStrandIDList(1);
    thisSeedStrand = Strands(thisSeedStrandID);
    thisStrandCurrent = thisSeedStrand.strandCurrent;
    ttt(thisSeedStrandID,baseStrandIter) = 0;
    pc(thisSeedStrandID,baseStrandIter) = thisStrandCurrent;
    startVertexVoltage = AllVoltages(thisSeedStrand.StartVertexIndex);
    endVertexVoltage = AllVoltages(thisSeedStrand.EndVertexIndex);
    if startVertexVoltage < endVertexVoltage,
        lowVoltageIndex = thisSeedStrand.StartVertexIndex;
    else%if startVertexVoltage < endVertexVoltage,
        lowVoltageIndex = thisSeedStrand.EndVertexIndex;
    end%if-else startVertexVoltage < endVertexVoltage,

    keepGoing = 1;
    while keepGoing==1,
        numSeedStrands = numel(seedStrandIDList);
        nextRoundSeedStrandIDList = zeros(1,numSeedStrands*5);
        nextRoundSeedCounter = 0;
        myVoltageList = zeros(1,numSeedStrands*5);
        for seedIter = 1:numSeedStrands,
            thisSeedStrandID = seedStrandIDList(seedIter);
            thisSeedStrand = Strands(thisSeedStrandID);
            motherTTT = ttt(thisSeedStrandID,baseStrandIter);
            motherPC = pc(thisSeedStrandID,baseStrandIter);
            thisSeedStrandTransitTime = transitTimeList(thisSeedStrandID); 
            %find low voltage node attached 
            startVertexVoltage = AllVoltages(thisSeedStrand.StartVertexIndex);
            endVertexVoltage = AllVoltages(thisSeedStrand.EndVertexIndex);
            if startVertexVoltage < endVertexVoltage,
                motherLowVoltageIndex = thisSeedStrand.StartVertexIndex;
            else%if startVertexVoltage < endVertexVoltage,
                motherLowVoltageIndex = thisSeedStrand.EndVertexIndex;
            end%if-else startVertexVoltage < endVertexVoltage,
            motherLowVoltage = AllVoltages(motherLowVoltageIndex);
            %Now find the strands attached to this lowVoltageIndex
            attachedStrandIDList = StrandIndices(motherLowVoltageIndex,:);
            attachedStrandIDList = attachedStrandIDList(attachedStrandIDList~=0);
            attachedStrandIDList = attachedStrandIDList(attachedStrandIDList~=thisSeedStrandID);
            numAttachedStrands = numel(attachedStrandIDList);
            %---
            daughterCounter = 0;
            daughterStrandIDList = zeros(numAttachedStrands,1);
            daughterCurrentList = zeros(numAttachedStrands,1);
            for attachedIter = 1:numAttachedStrands,
                %Check to see if the attached strand leads to a lower voltage
                attachedStrandID = attachedStrandIDList(attachedIter);
                attachedStrand = Strands(attachedStrandID);
                attachedStrandNodes = [attachedStrand.StartVertexIndex,attachedStrand.EndVertexIndex];
                attachedStrandFarNodeIndex = attachedStrandNodes(attachedStrandNodes~=motherLowVoltageIndex);
                attachedStrandFarVoltage = AllVoltages(attachedStrandFarNodeIndex);
                if attachedStrandFarVoltage < motherLowVoltage,
                    %if so, add the strand to a path
                    daughterCounter = daughterCounter + 1;
                    daughterStrandIDList(daughterCounter) = attachedStrandID;
                    daughterCurrentList(daughterCounter) = abs(attachedStrand.strandCurrent);
                end%if attachedStrandFarVoltage < motherLowVoltage,
            end%for attachedIter = 1:numAttachedStrands,
            daughterStrandIDList = daughterStrandIDList(1:daughterCounter);
            daughterCurrentList = daughterStrandIDList(1:daughterCounter);
            %-----
            %Calculate the Current distribution amongst daughters
           	numDaughters = numel(daughterStrandIDList);
            if numDaughters > 0,
                totalDaughterCurrent = sum(daughterCurrentList);
                for daughterIter = 1:numDaughters,
                    thisDaughterStrandID = daughterStrandIDList(daughterIter);
                    thisDaughterCurrent = daughterCurrentList(daughterIter);
                    daughterPC = motherPC * thisDaughterCurrent / totalDaughterCurrent;
                    pc(thisDaughterStrandID,baseStrandIter) = daughterPC;
                    ttt(thisDaughterStrandID,baseStrandIter) = motherTTT + thisSeedStrandTransitTime;
                    %do not continue path if contribution from this base to this strand is too small
                    if daughterPC/thisDaughterCurrent > minCurrentFractionLimit,
                        nextRoundSeedCounter = nextRoundSeedCounter + 1;
                        nextRoundSeedStrandIDList(nextRoundSeedCounter) = thisDaughterStrandID;
                        myVoltageList(nextRoundSeedCounter) = Strands(thisDaughterStrandID).MedianVoltage;
                    end%if daughterPC/thisDaughterCurrent > minCurrentFractionLimit,
                end%for daughterIter = 1:numDaughters,
            end%if numDaughters > 0
        end%for seedIter = 1:numSeedStrands,
        nextRoundSeedStrandIDList = nextRoundSeedStrandIDList(1:nextRoundSeedCounter);
        myVoltageList = myVoltageList(1:nextRoundSeedCounter);
        if numel(nextRoundSeedStrandIDList) == 0,
            keepGoing = 0;
        end
        seedStrandIDList = unique(nextRoundSeedStrandIDList);
    end%while keepGoing == 1,
    disp([num2str(baseStrandIter),' of ',num2str(numBaseStrands),' : ',datestr(now)]);
end%for baseStrandIter = 1:numBaseStrands,



%%

%Now, for each strand, get the weighted-average transit time for the strand from all
%sources.
for strandIter = 1:numStrands,
    thisStrandID = strandIter;
    thisStrandTTT = ttt(thisStrandID,:);
    thisStrandPC = pc(thisStrandID,:);
    activeFlag = find(thisStrandTTT~=inf);
    activeTTT = thisStrandTTT(activeFlag);
    activePC = thisStrandPC(activeFlag);
    weightedTransitTime = sum(activeTTT.*activePC)/sum(activePC);
    Strands(strandIter).meanTransitTime = weightedTransitTime;
end;


gds.Strands = Strands;
    