function [GapConnectionStruct] = LocalThresholdRelaxationConnectionPrime4(thresholdRelaxationParameters)

%% VesselRepresentations is a structure including:
% RawVesselVolumeTifFname          : raw vessel volume tif full filename (FITC - a DAPI)
% EnhancedVesselVolumeTifFname     : rod match filter-enhanced vessel volume tif full filename
% IsotropicVesselMask              : isotropic vessel mask's tif full filename
% VectorizedMV3DFilename           : amira's vectorized vessel .mv3d full filename
% thresholdRelaxationParameters          : all parameters specifying how to connect gaps
%    boundBoxMinDist               : an endpoint must be at least this distance from bounding box to connect
%    boundBoxDimXYZ                : the range of x,y,z for the vessel volume data
%    maxGapDist                 : 1pxl = 1um
%    minGapDist               : 0 (set to some nonzero value for studies of gap length)
%    hopislands                    : {0,1} (toggles whether islands can be connection candidates)
%    BBmargin                      : the margin of the bounding box about an endpoint to find connection candidates
%    minvvol                       : parameter to JointThresholdVesselMasks indicating smallest volume for vessel segment
%    morphfiltsz                   : parameter to JointThresholdVesselMasks indicating smallest volume for vessel segment

%% GapConnectionStruct
% GapConnectionStruct.strandSet            : The .mv3d file cast as a matlab data structure
% GapConnectionStruct.allconnectionptpairs : Indicates what pt indices in strandSet should be connected
% GapConnectionStruct.connectiontypes      : TR or NN (for Threshold Relaxation or Nearest Neighbor); indicates
%                                          : whether the connection pair was connected via a simple nearest neighbor
%                                          : method or the threshold relaxation method.
% GapConnectionStruct.ThreshRelax          : Contains 2 fields, "BridgingStrands" and "BridgingConfidences"
%                                          : Bridging strands is a cell array of ordered lists of 3D centerline coordinates
%                                          : Bridging confidences is the corresponding scalar confidence per strand

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.


%%




maskSize = thresholdRelaxationParameters.maskSize;
rootName = thresholdRelaxationParameters.rootName;
rootFolder = thresholdRelaxationParameters.rootFolder;
thresholdGapConfidence = thresholdRelaxationParameters.thresholdGapConfidence;
edgeBoundaryInVoxels = thresholdRelaxationParameters.edgeBoundaryInVoxels;
TRFilesFolder = thresholdRelaxationParameters.TRFilesFolder;
vascGrayscaleFileName = thresholdRelaxationParameters.vascGrayscaleFileName;
vascEnhancedGrayscaleFileName = thresholdRelaxationParameters.vascEnhancedGrayscaleFileName;
vascReconstructedMaskFileName = thresholdRelaxationParameters.vascReconstructedMaskFileName;
%-----
postNearestNeighborFileName = fullfile(TRFilesFolder,[rootName,'-postNearestNeighbor.mat']);
postIslandsFileName = fullfile(TRFilesFolder,[rootName,'-postIsland.mat']);



%% One can connect small gaps by pairing endpoints and testing for proximity and directionality
if exist(postNearestNeighborFileName,'file'),
    disp('Loading Previous Nearest Neighbor Results');
    load(postNearestNeighborFileName);
else%if exist(postNearestNeighborFileName,'file')
    disp(['Start Pair Nearest Neighbors : ',datestr(now,'mmddyy_HH:MM.SS')]);
    thresholdRelaxationParameters = PairNearestNeighborEndpointsPrime3(thresholdRelaxationParameters);
    save(postNearestNeighborFileName,'thresholdRelaxationParameters')
end%if-else exist(postNearestNeighborFileName,'file')


%% Get centerpoints of islands in mask
if exist(postIslandsFileName,'file'),
    disp('Loading Previous Island Points Results');
    load(postIslandsFileName)
else%if exist(postIslandsFileName,'file'),
    disp(['Start Get Island Points : ',datestr(now,'mmddyy_HH:MM.SS')]);
        thresholdRelaxationParameters = GetIslandPointsFromMaskPrime2(thresholdRelaxationParameters);
    % Augment strands with island centers
        thresholdRelaxationParameters = AugmentStrandsWithIslandsPrime2(thresholdRelaxationParameters);
    allVerts=thresholdRelaxationParameters.Vertices.AllVerts;
    save(postIslandsFileName,'thresholdRelaxationParameters','allVerts');
end%if-else exist(postIslandsFileName,'file'),    
disp(['Completed Island Points : ',datestr(now,'mmddyy_HH:MM.SS')]);

try
    connections_jk = thresholdRelaxationParameters.connections2add;
    connectionsretest_jk = thresholdRelaxationParameters.connections2retest;
catch
    disp('Loaded older-style post-island file');
end

%%

% get singly-unconnected endpoints from connectionsretest_jk and connections_jk
if isempty(connectionsretest_jk),
    connectionsretest_jk.pt2pt = [1 1];
end%if isempty(connectionsretest_jk),
%-----
if thresholdRelaxationParameters.PreprocessNNConnections    % If the small obvious gaps should be connected via NN, set this to 1
    singleuconepts=unique([connectionsretest_jk.pt2pt]);                    % WARNING: NN connections do not produce bridging strands
else%if thresholdRelaxationParameters.PreprocessNNConnections
    singleuconepts=unique([[connectionsretest_jk.pt2pt],[connections_jk.pt2pt]]);
end%if-else thresholdRelaxationParameters.PreprocessNNConnections


%% Preallocate
connectionsbyrelaxation=struct([]);                          % output structure
BridgeStrands=cell(1,numel(singleuconepts));                 % cell array of strand centerline points
GapConfidences = single(zeros(1,numel(singleuconepts)));             % confidence in each connection
medianGapConfidences = single(zeros(1,numel(singleuconepts)));             % confidence in each connection
% toggle display
dispon = 0;



% tmpresultsfname = ['partialresults-',datestr(clock,30),'.mat'];
tmpresultsfname = fullfile(rootFolder,[rootName,'-TRPartialResults.mat']);
tempResultsBackupFileName = fullfile(rootFolder,[rootName,'-TRPartialResultsBackup.mat']);
finalResultsFileName = fullfile(rootFolder,[rootName,'-TRFinalResults.mat']);
%-----
%Load the full masks here
loadedVariables = load(vascReconstructedMaskFileName);
variableNames = fieldnames(loadedVariables);
maskVariableName = variableNames{1};
fullB = loadedVariables.(maskVariableName);
clear loadedVariables variableNames maskVariableName;
fullB = struct2mask(fullB);
%--
% % % % % loadedVariables = load(vascGrayscaleFileName);
% % % % % variableNames = fieldnames(loadedVariables);
% % % % % maskVariableName = variableNames{1};
% % % % % fullR = loadedVariables.(maskVariableName);
% % % % % clear loadedVariables variableNames maskVariableName;
%--
loadedVariables = load(vascEnhancedGrayscaleFileName);
variableNames = fieldnames(loadedVariables);
maskVariableName = variableNames{1};
fullD = loadedVariables.(maskVariableName);
clear loadedVariables variableNames maskVariableName;
%-----


completedCycles = 0;
if exist(tmpresultsfname,'file');
    load(tmpresultsfname);
end
    
BBmargin = thresholdRelaxationParameters.BBmargin;
numBBmargin = size(BBmargin,2);

%noteToSelf: addded 08/23/10_PST
skipEdgeLimits = [1+edgeBoundaryInVoxels(1,1) ,  maskSize(1)-edgeBoundaryInVoxels(1,2) ;...
                1+edgeBoundaryInVoxels(2,1) ,  maskSize(2)-edgeBoundaryInVoxels(2,2) ;...
                1+edgeBoundaryInVoxels(3,1) ,  maskSize(3)-edgeBoundaryInVoxels(3,2)];
                
            
backupFileToggle = 1;%Hardcode
allVerts = thresholdRelaxationParameters.Vertices.AllVerts;




for iCONN = completedCycles+1:numel(singleuconepts)
    ptconn1=singleuconepts(iCONN);
    disp([allVerts(ptconn1,:)])
    mycoords = allVerts(ptconn1,:);        

    skipEdgeFlag = 0;
    if mycoords(1)<skipEdgeLimits(2,1), skipEdgeFlag = 1; end
    if mycoords(2)<skipEdgeLimits(1,1), skipEdgeFlag = 1; end
    if mycoords(3)<skipEdgeLimits(3,1), skipEdgeFlag = 1; end
    if mycoords(1)>skipEdgeLimits(2,2), skipEdgeFlag = 1; end
    if mycoords(2)>skipEdgeLimits(1,2), skipEdgeFlag = 1; end
    if mycoords(3)>skipEdgeLimits(3,2), skipEdgeFlag = 1; end


    if skipEdgeFlag == 1 ,
        BridgeStrands{iCONN} = [];
        GapConfidences(iCONN) = NaN;    
        disp('Skipping Edge Point');
    else
    
        BridgeStrands{iCONN} = [];
        GapConfidences(iCONN) = -999;
        for BBmarginIter = 1:numBBmargin,
            currentBBmargin = BBmargin(BBmarginIter);
            disp(['Analyzing candidate point number: ',num2str(iCONN),' of ',num2str(numel(singleuconepts)),': ',datestr(now,'mmddyy_HH:MM.SS')]);

            
            
% % % %             strand1=(augmentedStrandSet.ctrPoint2StrandLUT(ptconn1,2));
% % % %             s1inds=strandSet.ctrPoint2StrandLUT(:,2)==strand1;
            strand1 = thresholdRelaxationParameters.Vertices.StrandIndices(ptconn1,:);
            strand1 = strand1(strand1~=0);
            s1inds = thresholdRelaxationParameters.Strands(strand1).StartToEndIndices;

            % compute the bounding box that will ~minimally contain the strands associated with endpoints
            [vollimits] = round(getstrandBB2(allVerts(ptconn1,:),currentBBmargin));

            [vollimits] = boundscheck(vollimits,[1 1 1,maskSize(1),maskSize(2),maskSize(3)]);

            [y x z] = ndgrid(vollimits(1):vollimits(2), vollimits(3):vollimits(4), vollimits(5):vollimits(6));
            

            % set coords based on vollimits
            coords.x1min=vollimits(1);coords.x1max=vollimits(2);coords.x2min=vollimits(3);coords.x2max=vollimits(4);coords.x3min=vollimits(5);coords.x3max=vollimits(6);
            % get the local subvolume to render (B=binary mask, R=raw volume, D=enhanced volume)
            B = fullB(coords.x1min:coords.x1max,coords.x2min:coords.x2max,coords.x3min:coords.x3max);
%NoteToSelf:(hardcode): Raw has been replaced with Enhanced            
            R = fullD(coords.x1min:coords.x1max,coords.x2min:coords.x2max,coords.x3min:coords.x3max);
            D = fullD(coords.x1min:coords.x1max,coords.x2min:coords.x2max,coords.x3min:coords.x3max);

            
            

    % % % % %         if dispon
    % % % % %             % Draw binary mask in bounding box with strands
    % % % % %             figure;clf;hold on;
    % % % % %             patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    % % % % %             atmp = augmentedStrandSet;
    % % % % %             inbb1 = (vollimits(3)<= atmp.strandData(:,2)) & (vollimits(4)>= atmp.strandData(:,2));
    % % % % %             inbb2 = (vollimits(1)<= atmp.strandData(:,3)) & (vollimits(2)>= atmp.strandData(:,3));
    % % % % %             inbb3 = (vollimits(5)<= atmp.strandData(:,4)) & (vollimits(6)>= atmp.strandData(:,4));
    % % % % %             inbb = inbb1 & inbb2 & inbb3;
    % % % % %             atmp.strandData(~inbb,:) = NaN;
    % % % % %             jk_amira2mat_displayLabeledStrands(atmp);
    % % % % %             grid on;axis equal;
    % % % % %             axis(vollimits)
    % % % % %             axis off;
    % % % % %         end
    
    
            % Get intensity distribution of vessel along centerline points on strands associated with endpoints
            disp('Getting local intensity distribution');
            [LocalDist,Thresholds,FineThresholds] = GetLocalIntensityDistributionPrime(D,R,B);

            % Get disconnected endpoint strand masks (mask only on strands connected to endpoints)
            disp('Getting endpoint and strand masks');
            [EPStrandMask,EPMask] = GetSingleEndpointStrandMask2(B,x,y,z,allVerts(s1inds,:),allVerts(ptconn1,:));
            % Choose a new mask that connects the current endpoint to some other centerline point, return the candidate point
            % that connects ptconn1 (in ptconn2) and the endpoint location found (in ept2)
            disp('Finding connecting point via threshold relaxation...');
            [ptconn2,ept2,BacktrackMask] = ConnectedCenterlinePointThreshRelaxPrime2(D,B,EPStrandMask,EPMask,allVerts(ptconn1,:),allVerts,FineThresholds,x,y,z,thresholdRelaxationParameters.minvvol,thresholdRelaxationParameters.morphfiltsz);
            if ~isempty(ptconn2) && ~isempty(ept2)
% % %                 strand2=(augmentedStrandSet.ctrPoint2StrandLUT(ptconn2,2));
% % %                 s2inds=strandSet.ctrPoint2StrandLUT(:,2)==strand2;
                strand2 = thresholdRelaxationParameters.Vertices.StrandIndices(ptconn2,:);
                strand2 = strand2(strand2~=0);
                s2inds = thresholdRelaxationParameters.Strands(strand2).StartToEndIndices;


                disp('Finding the gap connecting mask...');
                [GapConnectingMask,GapConfidence] = ChooseGapConnectingMaskPrime(D,R,B,BacktrackMask,EPMask,allVerts(ptconn1,:),allVerts(ptconn2,:),Thresholds,x,y,z,thresholdRelaxationParameters.minvvol,thresholdRelaxationParameters.morphfiltsz);
                fprintf(1,'Gap confidence was %2.2f\n',GapConfidence);
                if ~isreal(GapConfidence)
                    disp('gap confidence is imaginary');
                    keyboard;
                end%if ~isreal(GapConfidence)

                IncompleteBridgingStrandComputation = false;
                if or(GapConfidence>thresholdGapConfidence ,  BBmarginIter == numBBmargin) && GapConfidence>0,
                    % Find the minimal path from EP1 to EP2 in GapConnectingMask
                    disp('Estimating the bridging strand...');
                    
                    
                    [BridgeStrandEP1EP2] = ShortestBridgeEP1ToEP2Prime2(allVerts(ptconn1,:),allVerts(ptconn2,:),GapConnectingMask,x,y,z);
                    % Save the strand and confidence in a cell array
                    BridgeStrands{iCONN}=BridgeStrandEP1EP2;
                    GapConfidences(iCONN)=GapConfidence;
                    
                    %-----
                    bridgeIdx = sub2ind(maskSize,BridgeStrandEP1EP2(:,1),BridgeStrandEP1EP2(:,2),BridgeStrandEP1EP2(:,3));
                    bridgeGray = fullD(bridgeIdx);
                    medianBridgeGray = median(bridgeGray);
                    polyfitINDs = Thresholds.Enhanced>1;
                    P1 = polyfit(log(Thresholds.Enhanced(polyfitINDs)),Thresholds.zrange(polyfitINDs),1);
                    medianGapConfidences(iCONN) = polyval(P1,log(medianBridgeGray));
                    %-----
                    
                    
                else%if or(GapConfidence>thresholdGapConfidence , and(GapConfidence>0 , BBmarginIter == numBBmargin)),
                    if BBmarginIter == numBBmargin,
                        disp('GAP CONFIDENCE BELOW ZERO at largest search volume.  Skipping.');
                    else%if BBmarginIter == numBBmargin,
                        disp('GAP COFIDENCE IS TOO LOW.  Attempting larger volume');
                    end%if-else BBmarginIter == numBBmargin,
                    BridgeStrands{iCONN} = [];
                    GapConfidences(iCONN) = 0;
                    medianGapConfidences(iCONN) = 0;
                end%if-else or(GapConfidence>thresholdGapConfidence ,  BBmarginIter == numBBmargin),

                
            else%if ~isempty(ptconn2) && ~isempty(ept2)
                % if no connection, save empty strands and confidences in a cell and structure array
                BridgeStrands{iCONN} = [];
                GapConfidences(iCONN) = -Inf;
                medianGapConfidences(iCONN) = -Inf;
            end%if-else ~isempty(ptconn2) && ~isempty(ept2)

    % % % % %         % surface render binary mask near endpoint connection
    % % % % %         if dispon
    % % % % %             % Draw binary mask in bounding box with strands with gap connecting mask and bridging strand
    % % % % %             h=figure;clf;hold on;
    % % % % %             patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    % % % % %             % draw strands associated with endpoints with solid black
    % % % % %             plot3(allVerts(s1inds,2),allVerts(s1inds,1),allVerts(s1inds,3),'k.-','linewidth',2,'markersize',15);
    % % % % %             % draw endpoints with solid black circles
    % % % % %             plot3(allVerts(ptconn1,2),allVerts(ptconn1,1),allVerts(ptconn1,3),'o','color',[.4 .4 0],'markersize',10);
    % % % % %             if ~isempty(ptconn2) && ~isempty(ept2)
    % % % % %                 % draw bridge strand with solid black line
    % % % % %                 plot3(allVerts(ptconn2,2),allVerts(ptconn2,1),allVerts(ptconn2,3),'ko','markersize',10);
    % % % % %                 patch(isosurface(x,y,z,GapConnectingMask, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
    % % % % %                 plot3(allVerts(s2inds,2),allVerts(s2inds,1),allVerts(s2inds,3),'k.-','linewidth',2,'markersize',15);
    % % % % %                 plot3(BridgeStrandEP1EP2(:,1),BridgeStrandEP1EP2(:,2),BridgeStrandEP1EP2(:,3),'m','linewidth',3);
    % % % % %                 plot3(ept2(2),ept2(1),ept2(3),'k.','markersize',10);
    % % % % %             end
    % % % % %             grid on;axis equal
    % % % % %             view(30,30)
    % % % % %             set(h,'position',[520   100   560   820])
    % % % % %             pause(1);
    % % % % %         end %if dispon
    % % % % %         close all;
            if (~isempty(ptconn2) && ~isempty(ept2)) && ~IncompleteBridgingStrandComputation
                connectionsbyrelaxation(iCONN).pt2pt = [ptconn1,ptconn2];
            else
                connectionsbyrelaxation(iCONN).pt2pt = [ptconn1,ptconn1];
            end
            %-----
            if GapConfidences(iCONN) < thresholdGapConfidence,
                disp('Confidence below threshold.  Repeating with larger block volume');
                disp(' ');
                disp(' ');
            else
                break%break out of for BBmarginIter = 1:numBBmargin loop
            end
        
        
        end%for BBmarginIter = 1:numBBmargin,
    
        if BBmarginIter==numBBmargin && isempty(BridgeStrands{iCONN}),
            disp(' ');
            disp(' ');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp('PossibleError - NoBridgingStrand - LocalThresholdRelaxationConnectionPrimeLine291');
            disp(' ');
            disp(' ');
        end%if BBmargin==numBBmargin && isempty(BridgeStrands{iCONN}),

    end%if-else skipEdgeFlag == 1
    
%     disp('check tmp2.mat if error stopped processing');
    completedCycles = iCONN;
    disp('saving temporary results...');
    switch backupFileToggle
        case 1,
            save(tmpresultsfname,'connectionsbyrelaxation','completedCycles','BridgeStrands','GapConfidences','medianGapConfidences');
        case 0,
            save(tempResultsBackupFileName,'connectionsbyrelaxation','completedCycles','BridgeStrands','GapConfidences','medianGapConfidences');
    end%switch backupFileToggle
    backupFileToggle = ~backupFileToggle;
    disp('..saved');
end%for iCONN = completedCycles+1:numel(singleuconepts)











clear R D B BacktrackMask EPMask EPStrandMask GapConnectingMask x y z
% Get strand length and pt-to-pt distance statistics
BridgeStrandStepLengths = zeros(1,numel(BridgeStrands));
BridgeStrandDists = zeros(1,numel(BridgeStrands));
for iBRS = 1:numel(BridgeStrands)
    if ~isempty(BridgeStrands{iBRS})
        BridgeStrandStepLengths(iBRS) = sum(sum(abs(diff(BridgeStrands{iBRS}))));
        BridgeStrandDists(iBRS) = sqrt(sum((BridgeStrands{iBRS}(1,:)-BridgeStrands{iBRS}(end,:)).^2));
    end
end
%% COLLECT AND ORGANIZE OUTPUT STRUCTURE DESCRIBING STRANDSET AND CONNECTIONS FOUND BY ALGORITHM
% Populate the returned strandset
% % GapConnectionStruct.strandSet = augmentedStrandSet;
if thresholdRelaxationParameters.PreprocessNNConnections  && ~isempty(connections_jk(1).pt2pt)
    % Populate the pairs of connected endpoints
    
    
    
    
    try
        GapConnectionStruct.allconnectionptpairs = [reshape([connections_jk.pt2pt],2,numel(connections_jk))';reshape([connectionsbyrelaxation.pt2pt],2,numel(connectionsbyrelaxation))'];
    catch
        GapConnectionStruct.allconnectionptpairs = [reshape([connections_jk.pt2pt],2,numel(connections_jk))';reshape([connectionsbyrelaxation.pt2pt],2,numel([connectionsbyrelaxation.pt2pt])/2)'];
        
    end
    
    
    
    % Populate the pairs of connected endpoints
    GapConnectionStruct.connectiontypes = [repmat('NN',numel(connections_jk),1);repmat('TR',numel(connectionsbyrelaxation),1)];
else
    % Populate the pairs of connected endpoints
    try
        GapConnectionStruct.allconnectionptpairs = reshape([connectionsbyrelaxation.pt2pt],2,numel(connectionsbyrelaxation))';
    catch
        GapConnectionStruct.allconnectionptpairs = reshape([connectionsbyrelaxation.pt2pt],2,numel([connectionsbyrelaxation.pt2pt])/2)';
    end
   
   % Populate the pairs of connected endpoints
    GapConnectionStruct.connectiontypes = repmat('TR',numel(connectionsbyrelaxation),1);
end

%%
%Add connecting strands for NN connections
strandCounter = size(BridgeStrands,2);
if thresholdRelaxationParameters.PreprocessNNConnections,  
    numConnectionsToAdd = size(connections_jk,2);
    if isempty(connections_jk(1).pt2pt), numConnectionsToAdd = 0;end
    for connectIter = 1:numConnectionsToAdd,
        %generate a straight line between the two points (m and n)
        %and assign it a new strandID
        try
            pointM = connections_jk(connectIter).coords(1,:) + [1 1 1];
            pointN = connections_jk(connectIter).coords(2,:) + [1 1 1];
        catch
            pointM = connections_jk(connectIter).coords(1,:) + int16([1 1 1]);
            pointN = connections_jk(connectIter).coords(2,:) + int16([1 1 1]);
        end%try-catch

        pointM(1:2) = fliplr(pointM(1:2));
        pointN(1:2) = fliplr(pointN(1:2));
        sepMN = pointN - pointM;
        maxSep = max(abs(sepMN));
        stepSize = sepMN/maxSep;
        currentPos = pointM;
        newCoordList = [];
        for stepIter = 1:maxSep-1,
            currentPos = currentPos + stepSize;
            newCoordList = [newCoordList;round(currentPos)];
        end%for stepIter = 1:maxSep-1,
        strandCounter = strandCounter + 1;
        BridgeStrands{strandCounter} = newCoordList;
        GapConfidences(strandCounter) = 99; %set confidence to 99 for NN strands
        medianGapConfidences(strandCounter) = 99;
    end%for connectIter = 1:numConnectionsToAdd,
end%if thresholdRelaxationParameters.PreprocessNNConnections,  


% Populate the bridging strand endpoints to add to the .mv3d file
GapConnectionStruct.maskSize = maskSize;
GapConnectionStruct.ThreshRelax.BridgingStrands = BridgeStrands;
GapConnectionStruct.ThreshRelax.BridgingConfidences = GapConfidences;
GapConnectionStruct.ThreshRelax.MedianBridgingConfidences = medianGapConfidences;

save (finalResultsFileName,'GapConnectionStruct');

