function blockingParameters = DetermineBlockingLimits(blockingParameters);
%This function determines the sub-blocking limits for a large block
%given a blockingParameters structure with the following fields:
%   fullDataSize : [y,x,z] size of full data to be subblocked
%   targetBlockSize : desired subBlock size (including overlap) - one # for all dimensions
%       note that final block size may slightly exceed the target size
%   overlapSize : the desired overlap between blocks - one # for all dimensions
%   fittingStyle : 'equal','trim', or 'pad'
%       equal or tight: (default), not all blocks will be equal size
%       trim: all blocks equal size, last few pixels may be lost
%       pad : all block equal size, last block end position may exceed data

%---
fullDataSize = blockingParameters.fullDataSize;
targetBlockSize = blockingParameters.targetBlockSize;
overlapSize = blockingParameters.overlapSize;
fittingStyle = blockingParameters.fittingStyle;
%---
%fittingStyle can be 'tight' or 'equal','trim', or 'pad'


%Determine Blocking Limits
numBlocks = ceil((fullDataSize+overlapSize)./targetBlockSize);
switch fittingStyle,
    case{'trim','Trim','TRIM'},
        blockStepSize = floor((fullDataSize - overlapSize)./numBlocks);
    otherwise,
        blockStepSize = ceil((fullDataSize - overlapSize)./numBlocks);
end%switch fittingStyle
blockY1List = linspace(1,1+blockStepSize(1)*(numBlocks(1)-1),numBlocks(1));
blockY2List = blockY1List + blockStepSize(1) + overlapSize;
blockX1List = linspace(1,1+blockStepSize(2)*(numBlocks(2)-1),numBlocks(2));
blockX2List = blockX1List + blockStepSize(2) + overlapSize;
blockZ1List = linspace(1,1+blockStepSize(3)*(numBlocks(3)-1),numBlocks(3));
blockZ2List = blockZ1List + blockStepSize(3) + overlapSize;
switch fittingStyle,
    case{'tight','Tight','TIGHT','Equal','equal','EQUAL'},
        blockY2List = min(blockY2List,fullDataSize(1));
        blockX2List = min(blockX2List,fullDataSize(2));
        blockZ2List = min(blockZ2List,fullDataSize(3));
end%switch fittingStyle

%If any dimension is only one block large, set that dimension to exactly the fullDataSize
if numBlocks(1) == 1, blockY2List = fullDataSize(1); end
if numBlocks(2) == 1, blockX2List = fullDataSize(2); end
if numBlocks(3) == 1, blockZ2List = fullDataSize(3); end


%Store block positions into a structure
blockCounter = 0;
blockList = [];
for yIter = 1:numBlocks(1),
    for xIter = 1:numBlocks(2),
        for zIter = 1:numBlocks(3),
            startPos = [blockY1List(yIter),blockX1List(xIter),blockZ1List(zIter)];
            stopPos = [blockY2List(yIter),blockX2List(xIter),blockZ2List(zIter)];
            blockSize = stopPos - startPos + 1; 
            blockCentroid = (1/2) * [startPos+stopPos];
            blockCounter = blockCounter + 1;
            blockList(blockCounter).blockID = blockCounter;
            blockList(blockCounter).startPos = startPos;
            blockList(blockCounter).stopPos = stopPos;
            blockList(blockCounter).blockSize = blockSize;
            blockList(blockCounter).blockCentroid = blockCentroid;
        end%for zIter;
    end%for xIter;
end%for yIter;

blockingParameters.blockList = blockList;

            
            
            
            
            






