function varargout = loadmovie(varargin);
%last revised 01/10/08 - Phil
%
%This function loads an entire image stack (stored as a multistack
%.tif file one frame at a time to avoid overflowing memory.
%
%Input arguments are stored in the structure myparameters as:
%   myparameters.input_filename =  path & filename string of desired file
%   myparameters.read_start = start frame to read
%   myparameters.read_stop = stop frame to read
%   myparameters.convert_type = uint8,uint16,int16,single,double(default = uint16)
%   myparemeters.convert_min = min value for conversion (default = 0)
%   myparameters.convert_max = max value for conversion (default = 2048)
%   myparameters.save_mat = 'yes', 'on', 'savemat' saves a .mat file
%   myparameters.waitbar_yn = 'yes' or 'no' to display waitbar
%   myparameters.window_prompt = 'Choose an Image file'
%   myparameters.cropRegion = [ystart, ystop, xstart, xstop, zstart,zstop];
%   myparameters.stepSize = [yStepSize, xStepSize, zStepSize];
%       use 0 for max/min limit

%Output argument is the 3D matrix containng thedata.
%If no input variable is given, a user input window displays.
%If the first input argument is not a structure, it is assumed to be a
%the input_filename string
%
%USE:
%MYPICS = LOADMOVIE;
%MYPICS = LOADMOVIE(INPUT_FILENAME, CROPREGION, CONVERT_TYPE, CONVERT_MIN, CONVERT_MAX, SAVE_MAT,WINDOW_PROMPT);
%MYPICS = LOADMOVIE(MYPARAMETERS);
%[MYPICS,MYPARAMETERS] = LOADMOVIE(MYPARAMETERS);
%[MYPICS,MYPARAMETERS] = LOADMOVIE('','','','','','','','yes','Prompt');
%   to convert to a .mat file
%
% Written by Phil Tsai 01/10/05



%EXTRACT INPUT PARAMETERS, IF THEY EXIST
nin = nargin;
nout = nargout;
myargin = varargin;
myargin{100} = '';
if nin >0,
    if isstruct(myargin{1}),
        myparameters = myargin{1};
    else%if isstruct(myargin{1})
        myparameters.input_filename = myargin{1};
        myparameters.cropRegion = myargin{2};
        myparameters.stepSize = myargin{3};
        myparameters.convert_type = myargin{4};
        myparameters.convert_min = myargin{5};
        myparameters.convert_max = myargin{6};
        myparameters.save_mat = myargin{7};
        myparameters.waitbar_yn = myargin{8};
        myparameters.window_prompt = myargin{9};
    end%if isstruct(myargin{1})
else%if nin>0
    myparameters = [];
end
input_filename = readparameters(myparameters,'input_filename','');
cropRegion = readparameters(myparameters,'cropRegion',[0 0 0 0 0 0]);
stepSize = readparameters(myparameters,'stepSize',[1 1 1]);
convert_type = readparameters(myparameters,'convert_type','uint16');
convert_min = readparameters(myparameters,'convert_min',0);
convert_max = readparameters(myparameters,'convert_max',2047);
save_mat = readparameters(myparameters,'save_mat','no');
waitbar_yn = readparameters(myparameters,'waitbar_yn','yes');
window_prompt = readparameters(myparameters,'window_prompt','Choose an image file');

start_frame = readparameters(myparameters,'start_frame',0);
stop_frame = readparameters(myparameters,'stop_frame',0);
if start_frame>0, read_start = start_frame; end
if stop_frame>0, read_stop = stop_frame; end

%DETERMINE FILENAME FOR OPENING
if isempty(input_filename),
    [filename,pathname] = uigetfile('*.tif',window_prompt);
    input_filename = [pathname,filename];
    myparameters.input_filename = filename;
    cd(pathname);
else%if isempty(input_filename)
    passed_path_filename = input_filename;
    [junk,filename_size] = size(passed_path_filename);
    endpath = max(findstr(passed_path_filename,filesep)); %location of last \ in path
    if isempty(endpath) %if only the filename alone was entered
        pathname = cd;
        input_filename = [pathname,filesep,passed_path_filename];
        filename = passed_path_filename;
    else%isempty(endpath)
        input_filename = passed_path_filename;
        filename = passed_path_filename(endpath+1:filename_size);
        pathname = passed_path_filename(1:endpath);
    end%if isemtpy(endpath)
end%if isempty(input_filename)

myinfo = imfinfo(input_filename);
numFrames = length(myinfo);
ySize = myinfo(1).Height;
xSize = myinfo(1).Width;
ColorType = myinfo(1).ColorType;
bitDepth = myinfo(1).BitDepth;
if bitDepth == 8, convert_type = 'uint8';end

if cropRegion(1)<1, yStart = 1; else yStart = cropRegion(1); end
if cropRegion(2)<1, yStop = ySize; else yStop = cropRegion(2); end
if cropRegion(3)<1, xStart = 1; else xStart = cropRegion(3); end
if cropRegion(4)<1, xStop = xSize; else xStop = cropRegion(4); end
if cropRegion(5)<1, zStart = 1; else zStart = cropRegion(5); end
if cropRegion(6)<1, zStop = numFrames; else zStop = cropRegion(6); end
yStepSize = stepSize(1);
xStepSize = stepSize(2);
zStepSize = stepSize(3);
calcYSize = length([yStart:yStepSize:yStop]);
calcXSize = length([xStart:xStepSize:xStop]);
calcZSize = length([zStart:zStepSize:zStop]);
matrixSize = [calcYSize,calcXSize,calcZSize];
rowLimits = [yStart yStepSize yStop];
colLimits = [xStart xStepSize xStop];


saveas_suffix = '.mat';
temp = size(filename);
temp2 = temp(2)-4;
temp3 = filename(1,1:temp2);
mat_filename = [temp3,saveas_suffix];
mat_pathname = pathname;
mat_fullname = [pathname,mat_filename];

switch convert_type
    case 'uint8', mybit = uint8(0); maxval = 255;
    case 'uint16', mybit = uint16(0); maxval = 2047;
    case 'int16', mybit = int16(0); maxval = 2047;
    case 'single', mybit = single(0); maxval = 2047;
    case 'double', mybit = double(0); maxval = 2047;
    otherwise, mybit = uint16(0);maxval=2047;
end

if strcmp(ColorType,'truecolor');
    matrixSize(3) = matrixSize(3)*3;
    mypics = repmat(mybit,matrixSize);
    RGB = 1;
else
    mypics = repmat(mybit,matrixSize);
    RGB = 0;
end


%-------------------------------
%-------------------------------
waitbar_yn = 'No';
%-------------------------------
%-------------------------------




waitbar_on = 0;
switch waitbar_yn
    case {'yes','y','Y','Yes','YES'}
        wb = waitbar(0,['Loading file ',filename,'...']);
        waitbar_on = 1;
end
















frameCounter = 0;
for k = zStart:zStepSize:zStop,
    temppic = imread(input_filename,k,'PixelRegion',{rowLimits,colLimits});
    convert_yn = readparameters(myparameters,'convert_type','no');
    frameCounter = frameCounter + 1;
    
    if ~strcmp(convert_yn,'no'),
        temppic = double(temppic);
        temppic = temppic-convert_min;
        temppic = max(temppic,0);
        temppic = maxval * temppic / (convert_max - convert_min);
    end
    if RGB == 0,
        mypics(:,:,frameCounter) = temppic;
    else
        mypics(:,:,frameCounter*3-2 : (k-zStart+1)*3) = temppic;
    end
    if waitbar_on == 1, waitbar((k-zStart)/(zStop+1-zStart),wb); end
end%for k = zStart:zStepSize:zStop,

if waitbar_on ==1, close(wb); end

switch save_mat,
    case {'on','yes','savemat'}
        save(mat_fullname,'mypics');
        myparameters.mat_filename = mat_filename;
        myparameters.mat_pathname = mat_pathname;
    otherwise
end


if RGB == 1,
    tempPics = mypics;
    matrixSize(3)=matrixSize(3)/3;
    mypics = zeros([matrixSize,3]);
    for k = 1:size(tempPics,3),
        mypics(:,:,ceil(k/3),mod(k-1,3)+1) = tempPics(:,:,k);
    end%for k = 1:size(tempPics,3),
end%if RGB == 1,

    
varargout{1} = mypics;
if nout >1,
    varargout{2} = myparameters;
end

return%function





function output_parameter = readparameters(paramstruct,parameter_name,default_value);
    if isfield(paramstruct,parameter_name),
        if ~isempty(paramstruct.(parameter_name))
            output_parameter = paramstruct.(parameter_name);
        else
            output_parameter = default_value;
        end
    else
        output_parameter = default_value;
    end
return
